<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00004004ReajusteContratoFormacaoPrecoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Sm00004004ReajusteContratoFormacaoPrecoService
{
    
    protected $reajusteContratoFactory;
    protected $connection;
    protected $environment;
    protected $calculados;
    
    public function __construct (Sm00004004ReajusteContratoFormacaoPrecoFactory $reajusteContratoFactory, EntityManager $entityManager,  Environment $environment, NewCode $newCode){
        
        $this->reajusteContratoFactory  = $reajusteContratoFactory;
        $this->connection               = $entityManager->getConnection();
        $this->environment              = $environment;
        $this->newCode                  = $newCode;

    }
    
    /**
    *  Função que salva reajuste de acordo com a rows selecionada
    */
    
    public function selecionaUltimoReajuste($nrseqcontratos, $valororiginais, $idtpajuste, $vrpercdescajuste, $nrorg, $cdoperador){
            foreach ($valororiginais as $valororiginal) {
                
                    $rows = $this->buscarUltimosReajustes($nrseqcontratos[0], $valororiginal, $nrorg);
                    $this->saveReajuste($rows[0], $idtpajuste, $vrpercdescajuste, $nrorg, $cdoperador);
                
            }
    }
    
    /*
    *  Prepara paramentros para busca pelo o ultimo reajuste de cada contrato
    */
    public function UltimosReajustesPorContrato($nrformprecoservico,$nrseqcontrato,$nrorg){
        $nrorg = $this->environment->getNrOrg();
       
        $params = array(
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG" => $nrorg
            );
        
        $return = $this->reajusteContratoFactory->UltimosReajustesPorContrato($params);
        
        return $return;
    }
     /*
    *  Prepara paramentros para busca pelo o ultimo reajuste de cada contrato
    */
    public function buscarFormacoesContratos($nrorg){
        $nrorg = $this->environment->getNrOrg();

        $params = array(
            "NRORG" => $nrorg
            );

        $return = $this->reajusteContratoFactory->buscarFormacoesContratos($params);

        return $return;
    }
    
    /**
    *  Busca pelo o ultimo reajuste de cada contrato
    */
    
    public function buscarUltimosReajustes($nrseqcontrato, $valororiginal, $nrorg){
    try{
        $params = array(
            "CONTRATO"=> $nrseqcontrato,
            "VALORORIGINAL"=> $valororiginal,
            "NRORG"=> $nrorg
            ); 

            return $this->reajusteContratoFactory->buscarUltimosReajustes($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Realiza o calculo de reajuste antes de salvar
    *  Função que salva reajuste de acordo com a rows selecionada
    *  Função que salva  um registro de reajuste na tabela de reajuste
    */
     
    public function saveReajuste($row, $idtpajuste, $vrpercdescajuste, $nrorg, $cdoperador){
    try {
          
        $valorpai = $row["VALORPAI"];
        $codigo = $row["NRREAJUSCONTFORMPREC"];
        $nrseqcontrato = $row["NRSEQCONTRATO"];
        $nritemocontrato = $row["NRITEMOCONTRATO"];
        $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
        $dataDoAjuste = date_create();
        $dataDoAjuste = $dataDoAjuste->format("d/m/Y H:i:s");
        
        $params = array("NRFORMPRECOSERVICO" => $nrformprecoservico,
                        "NRORG" => $nrorg
        );
        
        $param = array("NRSEQCONTRATO" => $nrseqcontrato,
                        "NRFORMPRECOSERVICO" => $nrformprecoservico,
                        "NRORG" => $nrorg
        );
        if($valorpai == null){
            $rowsParaAjuste = $this->reajusteContratoFactory->bucarRowsValorFixo($params);
        }else{
            $rowsParaAjuste = $this->reajusteContratoFactory->bucarRowsUltimoReajuste($param);
        }
        
        $nrreajuscontformprec = $this->newCode->getCodeByTableName("FCT_REAJUSCONTFORMPREC", $nrorg, 1, 10, true);
        $this->formaParams($nrseqcontrato, $nritemocontrato, $nrformprecoservico, $nrreajuscontformprec, $nrorg);
        
        $params = array(
            "NRREAJUSCONTFORMPREC" => $nrreajuscontformprec,
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG" => $nrorg,
            "IDATIVO" => "S",
            "DTINCLUSAO" => $dataDoAjuste
            ); 
       

        $this->reajusteContratoFactory->saveReajuste($params);
     
        
        $entrou = false;
        foreach ($rowsParaAjuste as $rowParaAjustar) {
            $nrcalcreajformpre = $this->newCode->getCodeByTableName("FCT_CALCREAJFORMPREC", $nrorg, 1, 10, true);
            if($valorpai == null){
                $codigopai = $rowParaAjustar["NRCALCFORMPRECSERVPAI"];
            }else{
                $codigopai = $rowParaAjustar["NRCALCREAJFORMPREPAI"];
                $nrcalcreajformpreAnterior = $rowParaAjustar["NRCALCREAJFORMPRE"];
            }
            
            
            if($rowParaAjustar["IDTIPOCALCULO"] == "V"){
                
                if($idtpajuste == "A"){
                $valorAjustado = $rowParaAjustar["VRUNITARIO"] + ($rowParaAjustar["VRUNITARIO"]*$vrpercdescajuste/100);
                }else
                if($idtpajuste == "R"){
                $valorAjustado = $rowParaAjustar["VRUNITARIO"] - ($rowParaAjustar["VRUNITARIO"]*$vrpercdescajuste/100);
                }
                    $valorUnitario = $valorAjustado;
                    $valorTotal = $valorAjustado;
                    $entrou = true;
            }else{
                
                $valorUnitario = $rowParaAjustar["VRUNITARIO"];
                    $valorTotal = $rowParaAjustar["VRTOTAL"];                     
            }
            
            $params = array("NRCALCREAJFORMPRE" =>$nrcalcreajformpre,
                            "NRREAJUSCONTFORMPREC" =>$nrreajuscontformprec,
                                "IDORDENACAO" =>$rowParaAjustar["IDORDENACAO"],
                                "IDITEMCALCULO" =>$rowParaAjustar["IDITEMCALCULO"],
                                "IDELEMENTO" =>$rowParaAjustar["IDELEMENTO"],
                                "NRCALCREAJFORMPREPAI" =>$codigopai,
                                "DSREFECALCULO" =>$rowParaAjustar["DSREFECALCULO"],
                                "CDHABILITSERV" =>$rowParaAjustar["CDHABILITSERV"],
                                "IDTIPOCALCULO" =>$rowParaAjustar["IDTIPOCALCULO"],
                                "DSEXPRESSAOCALC" =>$rowParaAjustar["DSEXPRESSAOCALC"],
                                "VRPERCCALCULO" =>$rowParaAjustar["VRPERCCALCULO"],
                                "VRUNITARIO" =>$valorUnitario,
                                "QTCALCULO" =>$rowParaAjustar["QTCALCULO"],
                                "IDUSAMEDIDA" =>$rowParaAjustar["IDUSAMEDIDA"],
                                "CDTPMEDIDASERV" =>$rowParaAjustar["CDTPMEDIDASERV"],
                                "QTMEDIDACALC" =>$rowParaAjustar["QTMEDIDACALC"],
                                "VRTOTAL" =>$valorTotal,
                                "IDTIPOREAJUSTE" =>$idtpajuste,
                                "VRPERCREAJUSTE" =>$vrpercdescajuste,
                                "VRTOTALANTE" =>$rowParaAjustar["VRUNITARIO"],
                                "NRORG" =>$rowParaAjustar["NRORG"],
                                "IDATIVO" =>$rowParaAjustar["IDATIVO"],
                                "DTINCLUSAO" =>$dataDoAjuste,
                                "DTULTATU" =>$rowParaAjustar["DTULTATU"],
                                "NRORGINCLUSAO" =>$rowParaAjustar["NRORGINCLUSAO"],
                                "CDOPERINCLUSAO" =>$cdoperador,
                                "NRORGULTATU" =>$rowParaAjustar["NRORGULTATU"],
                                "CDOPERULTATU" =>$rowParaAjustar["CDOPERULTATU"]
                );    
            $this->reajusteContratoFactory->saveReajusteFormacaoPreco($params);
            } 
            $this->alteraPais($nrreajuscontformprec, $nrorg, $nrformprecoservico, $valorpai, $codigo);
            $this->recalcularArvore($nrreajuscontformprec); 
        }
        catch (\Exception $e) {
            throw $e;   
        }
    }
    
    /**
    *  Prepara o save dos reajuste de acordo com o mnemônico pra o tipo valor fixo
    *  Chama as rotinas que salva  e aplica reajuste de acordo com a rows selecionada
    */
    
    public function saveReajusteAutomatico($nrreajuscontformprec, $nrformprecoservico, $tabela, $formacaoreajustes, $idtpajuste, $vrpercdescajuste, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador){
    try {    
            $escolhidos = array();
            foreach($formacaoreajustes as $formacaoreajuste){
                $formReaj = explode("-", $formacaoreajuste);
                $formacaoformReaj = intval($formReaj[0]);
                $iditemcalculo = $formReaj[1];
                
                    if($formacaoformReaj == $nrreajuscontformprec){
                        array_push($escolhidos,$iditemcalculo);
                    }
            }
            
            $this->saveReajusteElemento($nrformprecoservico, $escolhidos, $tabela, $idtpajuste, $vrpercdescajuste, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador);
            $this->aplicarReajuste($nrformprecoservico, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador, $nrreajuscontformprec, $tabela);
        }
        catch (\Exception $e) {
            throw $e;   
        }
    }
     
    /**
    *  Realiza o calculo de reajuste por elemento antes de salvar
    *  Função que salva reajuste por elemento  de acordo com a rows selecionada
    *  Função que salva  um registro de reajuste na tabela de reajuste
    */
    
    public function saveReajusteElemento($nrformprecoservico, $escolhidos, $tabela, $idtpajuste, $vrpercdescajuste, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador){
    try { 
        $dataDoAjuste = date_create();
        $dataDoAjuste = $dataDoAjuste->format("d/m/Y H:i:s");
        if($tabela == 2){
            $params = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRFORMPRECOSERVICO" => $nrformprecoservico,
                "NRORG" => $nrorg
            );
         }else{
            $params = array(
                "NRFORMPRECOSERVICO" => $nrformprecoservico,
                "NRORG" => $nrorg
            );
         }
        if($tabela == 2){
        $rowsParaAjuste = $this->reajusteContratoFactory->bucarRowsUltimoReajuste($params);
        }else{
        $rowsParaAjuste = $this->reajusteContratoFactory->bucarRowsValorFixo($params);
        }
        foreach ($rowsParaAjuste as $rowParaAjustar) {
             if($tabela == 1){
                $codigopai = $rowParaAjustar["NRCALCFORMPRECSERVPAI"];
                $nrcalcreajformpreAnterior = $rowParaAjustar["NRCALCFORMPRECSERV"];
                $nrreajuscontformprecTemp = null;
            }else{
                $codigopai = $rowParaAjustar["NRCALCREAJFORMPREPAI"];
                $nrcalcreajformpreAnterior = $rowParaAjustar["NRCALCREAJFORMPRE"];
                $nrreajuscontformprecTemp = $rowParaAjustar["NRREAJUSCONTFORMPREC"];
            }
           if (in_array($rowParaAjustar["IDITEMCALCULO"], $escolhidos)) {
                    if($rowParaAjustar["IDTIPOCALCULO"] == "V"){
                        
                        if($idtpajuste == "A"){
                        $valorAjustado = $rowParaAjustar["VRUNITARIO"] + ($rowParaAjustar["VRUNITARIO"]*$vrpercdescajuste/100);
                        }else
                        if($idtpajuste == "R"){
                        $valorAjustado = $rowParaAjustar["VRUNITARIO"] - ($rowParaAjustar["VRUNITARIO"]*$vrpercdescajuste/100);
                        }
                            $valorUnitario = $valorAjustado;
                            $valorTotal = $valorAjustado;
                    }else{
                        $valorUnitario = $rowParaAjustar["VRUNITARIO"];
                            $valorTotal = $rowParaAjustar["VRTOTAL"];
                    }
            }else{
                    $valorUnitario = $rowParaAjustar["VRUNITARIO"];
                    $valorTotal = $rowParaAjustar["VRTOTAL"];
                }
            $params = array(
                            "NRSEQCONTRATO" =>$nrseqcontrato,
                            "NRFORMPRECOSERVICO" =>$nrformprecoservico,
                            "NRITEMOCONTRATO" =>$nritemocontrato,
                            "IDORDENACAO" =>$rowParaAjustar["IDORDENACAO"],
                            "IDITEMCALCULO" =>$rowParaAjustar["IDITEMCALCULO"],
                            "IDELEMENTO" =>$rowParaAjustar["IDELEMENTO"],
                            "NRCALCREAJFORMPREPAI" =>$codigopai,
                            "DSREFECALCULO" =>$rowParaAjustar["DSREFECALCULO"],
                            "CDHABILITSERV" =>$rowParaAjustar["CDHABILITSERV"],
                            "IDTIPOCALCULO" =>$rowParaAjustar["IDTIPOCALCULO"],
                            "DSEXPRESSAOCALC" =>$rowParaAjustar["DSEXPRESSAOCALC"],
                            "VRPERCCALCULO" =>$rowParaAjustar["VRPERCCALCULO"],
                            "VRUNITARIO" =>$valorUnitario,
                            "QTCALCULO" =>$rowParaAjustar["QTCALCULO"],
                            "IDUSAMEDIDA" =>$rowParaAjustar["IDUSAMEDIDA"],
                            "CDTPMEDIDASERV" =>$rowParaAjustar["CDTPMEDIDASERV"],
                            "QTMEDIDACALC" =>$rowParaAjustar["QTMEDIDACALC"],
                            "VRTOTAL" =>$valorTotal,
                            "IDTIPOREAJUSTE" =>$idtpajuste,
                            "VRPERCREAJUSTE" =>$vrpercdescajuste,
                            "VRTOTALANTE" =>$rowParaAjustar["VRUNITARIO"],
                            "NRORG" =>$rowParaAjustar["NRORG"],
                            "IDATIVO" =>$rowParaAjustar["IDATIVO"],
                            "DTINCLUSAO" =>$dataDoAjuste,
                            "DTULTATU" =>$rowParaAjustar["DTULTATU"],
                            "NRORGINCLUSAO" =>$rowParaAjustar["NRORGINCLUSAO"],
                            "CDOPERINCLUSAO" =>$cdoperador,
                            "NRORGULTATU" =>$rowParaAjustar["NRORGULTATU"],
                            "CDOPERULTATU" =>$rowParaAjustar["CDOPERULTATU"],
                            "NRCALCREAJFORMPRE" =>$nrcalcreajformpreAnterior,
                            "NRREAJUSCONTFORMPREC" =>$nrreajuscontformprecTemp
                ); 
               
            $this->reajusteContratoFactory->saveReajusteElementoTemp($params);
            
            }
        }
        catch (\Exception $e) {
            throw $e;   
        }
    }
    
    /**
    *  Função que salva reajuste por elemento  que se encontrra na tabela temporária
    *  Função que salva  um registro de reajuste na tabela de reajuste
    *  Chama função que deleta o reajuste que se encontrra na tabela temporária
    */
    
    public function aplicarReajuste($nrformprecoservico, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador, $codigoUltimo, $tabela){
    $this->connection->beginTransaction();
        try{
            $dataDoAjuste = date_create();
            $dataDoAjuste = $dataDoAjuste->format("d/m/Y H:i:s");
            $paramsTemp = array(
                    "NRSEQCONTRATO" => $nrseqcontrato,
                    "NRFORMPRECOSERVICO" => $nrformprecoservico,
                    "NRORG" => $nrorg
                    );     
            $reajusteTempRows = $this->reajusteContratoFactory->bucarCalcTemp($paramsTemp);
            
            $nrreajuscontformprec = $this->newCode->getCodeByTableName("FCT_REAJUSCONTFORMPREC", $nrorg, 1, 10, true);
            
            $this->formaParams($nrseqcontrato, $nritemocontrato, $nrformprecoservico, $nrreajuscontformprec, $nrorg);
            
             $params = array(
            "NRREAJUSCONTFORMPREC" => $nrreajuscontformprec,
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG" => $nrorg,
            "IDATIVO" => "S",
            "DTINCLUSAO" => $dataDoAjuste
            ); 
            $this->reajusteContratoFactory->saveReajuste($params);
            
            foreach ($reajusteTempRows as $reajusteTempRow) {
                $nrcalcreajformpre = $this->newCode->getCodeByTableName("FCT_CALCREAJFORMPREC", $nrorg, 1, 10, true);
                
                $tipoReajuste = $reajusteTempRow["IDTIPOREAJUSTE"];
                
                $params = array("NRCALCREAJFORMPRE" =>$nrcalcreajformpre,
                                "NRREAJUSCONTFORMPREC" =>$nrreajuscontformprec,
                                "IDORDENACAO" =>$reajusteTempRow["IDORDENACAO"],
                                "IDTIPOREAJUSTE" =>$tipoReajuste,
                                "IDITEMCALCULO" =>$reajusteTempRow["IDITEMCALCULO"],
                                "IDELEMENTO" =>$reajusteTempRow["IDELEMENTO"],
                                "NRCALCREAJFORMPREPAI" =>$reajusteTempRow["NRCALCREAJFORMPREPAI"],
                                "DSREFECALCULO" =>$reajusteTempRow["DSREFECALCULO"],
                                "CDHABILITSERV" =>$reajusteTempRow["CDHABILITSERV"],
                                "IDTIPOCALCULO" =>$reajusteTempRow["IDTIPOCALCULO"],
                                "DSEXPRESSAOCALC" =>$reajusteTempRow["DSEXPRESSAOCALC"],
                                "VRPERCCALCULO" =>$reajusteTempRow["VRPERCCALCULO"],
                                "VRUNITARIO" =>$reajusteTempRow["VRUNITARIO"],
                                "QTCALCULO" =>$reajusteTempRow["QTCALCULO"],
                                "IDUSAMEDIDA" =>$reajusteTempRow["IDUSAMEDIDA"],
                                "CDTPMEDIDASERV" =>$reajusteTempRow["CDTPMEDIDASERV"],
                                "QTMEDIDACALC" =>$reajusteTempRow["QTMEDIDACALC"],
                                "VRTOTAL" =>$reajusteTempRow["VRTOTAL"],
                                "VRPERCREAJUSTE" =>$reajusteTempRow["VRPERCREAJUSTE"],
                                "VRTOTALANTE" =>$reajusteTempRow["VRUNITARIO"],
                                "NRORG" =>$reajusteTempRow["NRORG"],
                                "IDATIVO" =>$reajusteTempRow["IDATIVO"],
                                "DTINCLUSAO" =>$reajusteTempRow["DTINCLUSAO"],
                                "DTULTATU" =>$reajusteTempRow["DTULTATU"],
                                "NRORGINCLUSAO" =>$reajusteTempRow["NRORGINCLUSAO"],
                                "CDOPERINCLUSAO" =>$reajusteTempRow["CDOPERINCLUSAO"],
                                "NRORGULTATU" =>$reajusteTempRow["NRORGULTATU"],
                                "CDOPERULTATU" =>$reajusteTempRow["CDOPERULTATU"]
                                );
                $this->reajusteContratoFactory->saveReajusteFormacaoPreco($params);
            }
            $this->reajusteContratoFactory->deleteCalcTemp($paramsTemp);
            $this->alteraPaisUtomatico($nrreajuscontformprec, $nrorg, $nrformprecoservico, $codigoUltimo, $tabela);
            $this->recalcularArvore($nrreajuscontformprec); 
            
            $this->connection->commit();
            return 0;
        }catch(\Exception $e){
            $this-> connection->rollback();
            throw $e;
        }
            
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
    
    public function removerAjusteTemp($nrorg){
        $this->connection->beginTransaction();
        try{
        $params = array(
            "NRORG"=> $nrorg
            ); 

            $this->reajusteContratoFactory->removerAjusteTemp($params);
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
     /**
    *  Realiza a organização de cada row pái de acordo com a row filha
    */
    
    public function alteraPaisUtomatico($nrreajuscontformprec, $nrorg, $nrformprecoservico, $codigoUltimo, $tabela){
        
         if($tabela == 2){
             $pais = $this->buscarCodPais($codigoUltimo, $nrorg);
         }else{
             $pais = $this->buscarCodPais($nrreajuscontformprec, $nrorg);
         }
         
        foreach ($pais as $pai) {
             
            if($tabela == 2){
                
              $idItemFormAux = $this->buscarIditemFormUltimoreajuste($codigoUltimo, $pai["NRCALCREAJFORMPREPAI"], $nrorg);
            }else{
              $idItemFormAux = $this->buscarIditemForm($nrformprecoservico, $pai["NRCALCREAJFORMPREPAI"], $nrorg);
            }
              
              $idItemForm = $idItemFormAux[0]["IDITEMCALCULO"];
               
            $params = array(
            "NRORG"=> $nrorg,
            "NRFORMPRECOSERVICO"=> $nrformprecoservico
            ); 
            $codigo = $this->reajusteContratoFactory->buscarUltimoCodigo($params);
            $codigo = $codigo[0]["ULTIMOCODIGO"];
            if($tabela == 2){
              $nrPaiCorretoAux = $this->buscarNrPaiCorreto($codigo, $idItemForm, $nrorg);
            }else{
              $nrPaiCorretoAux = $this->buscarNrPaiCorreto($nrreajuscontformprec, $idItemForm, $nrorg);
                }
              $nrPaiCorreto = $nrPaiCorretoAux[0]["NRCALCREAJFORMPRE"];
              $this->atualizaCodPai($nrPaiCorreto, $nrreajuscontformprec, $pai["NRCALCREAJFORMPREPAI"], $nrorg); 
        };
        
    }

    /**
    *  Realiza a organização de cada row pái de acordo com a row filha
    */
    
    public function alteraPais($nrreajuscontformprec, $nrorg, $nrformprecoservico, $valorpai, $codigoAtual){
        $params = array(
            "NRORG"=> $nrorg,
            "NRFORMPRECOSERVICO"=> $nrformprecoservico
            ); 
        $codigo = $this->reajusteContratoFactory->buscarUltimoCodigo($params);
        $codigo = $codigo[0]["ULTIMOCODIGO"]-1;
        $pais = $this->buscarCodPais($nrreajuscontformprec, $nrorg);
        
        foreach ($pais as $pai) {
             
            if($valorpai == null){
                $idItemFormAux = $this->buscarIditemForm($nrformprecoservico, $pai["NRCALCREAJFORMPREPAI"], $nrorg);
              }else{
                $idItemFormAux = $this->buscarIditemFormUltimoreajuste($codigoAtual, $pai["NRCALCREAJFORMPREPAI"], $nrorg);
              }
              $idItemForm = $idItemFormAux[0]["IDITEMCALCULO"];
              $nrPaiCorretoAux = $this->buscarNrPaiCorreto($nrreajuscontformprec, $idItemForm, $nrorg);
              
              $nrPaiCorreto = $nrPaiCorretoAux[0]["NRCALCREAJFORMPRE"];
              
              $this->atualizaCodPai($nrPaiCorreto, $nrreajuscontformprec, $pai["NRCALCREAJFORMPREPAI"], $nrorg); 
        };
        
    }
    
    /**
    *  Busca o código de cada row no banco
    */
    
    public function buscarCodPais($nrreajuscontformprec, $nrorg){

        try{
        $params = array(
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRORG"=> $nrorg
            ); 

            return $this->reajusteContratoFactory->buscarCodPais($params);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Busca o id entificador de cada item de cada row no banco
    */
    
    public function buscarIditemForm($nrformprecoservico, $nrcalcformprecserv, $nrorg){

        try{
        $params = array(
            "NRFORMPRECOSERVICO"=> $nrformprecoservico,
            "NRCALCFORMPRECSERV"=> $nrcalcformprecserv,
            "NRORG"=> $nrorg
            ); 

           return $this->reajusteContratoFactory->buscarIditemForm($params);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
     /**
    *  Busca o id entificador de cada item de cada row no banco
    */
    
    public function buscarIditemFormUltimoreajuste($nrreajuscontformprec, $nrcalcformprecserv, $nrorg){

        try{
        $params = array(
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRCALCREAJFORMPRE"=> $nrcalcformprecserv,
            "NRORG"=> $nrorg
            ); 

           return $this->reajusteContratoFactory->buscarIditemFormUltimoreajuste($params);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Busca o o numero correto de cada item de cada row no banco
    */
    
    public function buscarNrPaiCorreto($nrreajuscontformprec, $iditemcalculo, $nrorg){

        try{
        $params = array(
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "IDITEMCALCULO"=> $iditemcalculo,
            "NRORG"=> $nrorg
            ); 

          return $this->reajusteContratoFactory->buscarNrPaiCorreto($params);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Atualiza o numeros de cada rows filha do contrato no banco
    */
    
    public function atualizaCodPai($nrPaiCorreto, $nrreajuscontformprec, $nrcalcformprecservpai, $nrorg){

        try{
        $params = array(
            "NOVOPAI"=> $nrPaiCorreto,
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRCALCREAJFORMPREPAI"=> $nrcalcformprecservpai,
            "NRORG"=> $nrorg
            ); 

            $this->reajusteContratoFactory->atualizaCodPai($params);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
  
    public function formaParams($nrseqcontrato, $nritemocontrato, $nrformprecoservico, $nrreajuscontformprec, $nrorg){
        try{
        $params = array(
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRFORMPRECOSERVICO"=> $nrformprecoservico,
            "NRSEQCONTRATO"=> $nrseqcontrato,
            "NRITEMOCONTRATO"=> $nritemocontrato,
            "NRORG"=> $nrorg
            ); 
            
            $this->reajusteContratoFactory->atualizaNumeroDoAjuste($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
   
    public function buscarPaiMaster($nrreajuscontformprec, $nrorg){
        try{
        $params = array(
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRORG"=> $nrorg
            ); 
            
            return $this->reajusteContratoFactory->buscarPaiMaster($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
    
    public function buscarIndes($munero, $nrorg){
        try{
        $params = array(
            "NRCALCFORMPRECSERVPAI"=> $munero,
            "NRORG"=> $nrorg
            ); 
            
            return $this->reajusteContratoFactory->buscarIndes($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
    
    public function buscarIdFormação($paiMaster, $nrorg){
        try{
        $params = array(
            "NRCALCREAJFORMPRE"=> $paiMaster,
            "NRORG"=> $nrorg
            ); 
            
            return $this->reajusteContratoFactory->buscarIdFormação($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *  Prepara os parâmetros e chama a função no Factory
    */
    
    public function buscarNuCalculoReajust($iditemcalculo,$nrreajuscontformprec, $nrorg){
        try{
        $params = array(
            "IDITEMCALCULO"=> $iditemcalculo,
            "NRREAJUSCONTFORMPREC"=> $nrreajuscontformprec,
            "NRORG"=> $nrorg
            ); 
            
            return $this->reajusteContratoFactory->buscarNuCalculoReajust($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * A função "insereItemFormPrec" prepara o array para inserção dos itens de contrato 
     * e centraliza as funções de inserção com soma ou separadamente.  
     */
    
    public function insereItemFormPrec($servicos, $soma, $proposta){
          
        //unset($servicos["NRFORMPRECOSERVICO"]);
        unset($servicos["NMFORMPRECOSERVICO"]);
        unset($servicos["CDSERVORCAFORM"]);
        unset($servicos["NRCALCFORMPRECSERV"]);
        unset($servicos["NMELEMENTO"]);
        unset($servicos["CDHABILITSERV"]);
        unset($servicos["changed"]);
        
        if ($soma == true){
            $this->insereItFormSoma($servicos, $soma, $proposta);
        }
        else
        {
            $this->insereItForm($servicos, $proposta); 
        }
        
    }

    /**
     * A função "insereItFormSoma" insere os itens de contrato quando escolhida 
     * a opção "Somar os Valores".
     */
    
    public function insereItFormSoma($itens, $soma, $proposta){
        
       $filtro = array();
       $arrNewcode = array();
       $nrorg = $this->environment->getNrOrg();
       $operador = $this->environment->getCdOperador();
       
       $this->connection->beginTransaction();
      
       try {
            /* Filtra os indices dos itens que possuem o mesmo serviço */
            asort($itens["CDSERVORCA"]);
        
            foreach ($itens["CDSERVORCA"] as $chave => $valor) {
                $filtro[] = $chave;
            }
            
            $count = count($itens["CDSERVORCA"]);
            
            /*Com os indices filtrados, insere os valores na tabela FCT_ITEMORCAMENTO */
            for ($i = 0; $i < $count; $i++) {
                 
                $ind =  $filtro[$i];
                
                /* Se for o primeiro índice, semore insere */ 
                if ($i == 0){
                    
                    $novoCodigo = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
                    
                    $arrNewCode[] = $novoCodigo;
                     
                    $params = array("NRITEMORCAMENTO"  => $novoCodigo, 
                                    "NRSEQORCAMENTO"   => $proposta, 
                                    "CDSERVORCA"       => $itens["CDSERVORCA"][$ind], 
                                    "NMITEMORCAMENTO"  => $itens["NMSERVORCA"][$ind],
                                    "QTITEMORCAMENTO"  => $itens["QTCALCULO"][$ind], 
                                    "CDTPMEDIDASERV"   => $itens["CDTPMEDIDASERV"][$ind], 
                                    "QTMEDIDAORCA"     => $itens["QTMEDIDACALC"][$ind],
                                    "VRUNITITEMORCA"   => $itens["VRUNITARIO"][$ind], 
                                    "VRTOTALITEMORCA"  => $itens["VRTOTAL"][$ind], 
                                    "IDMULTMEDVRTOT"   => $itens["IDUSAMEDIDA"][$ind], 
                                    "NRORG"            => $nrorg,
                                    "CDOPERADOR"       => $operador
                    );
                     
                    $this->factoryItFormPrecProp->insereItemContFormPreco($params);
                    $this->inserirRelacionamentoFormacaoPrecoProposta($itens["NRFORMPRECOSERVICO"][$ind], $novoCodigo, $proposta, $nrorg, $operador);
                }
                /*Após o primeiro indice, insere ou edita, de acordo com a regra abaixo*/
                else {
                    
                    $ante = $i - 1;
                    $indA = $filtro[$ante]; 
                    /* Se o serviço do índice atual for igual ao índice anterior, edita os valores do item. */
                    if ($itens["CDSERVORCA"][$ind] == $itens["CDSERVORCA"][$indA]){
                         
                        $params = array("VRTOTALITEMORCA"  =>  $itens["VRTOTAL"][$ind],
                                        "NRITEMORCAMENTO"  =>  $novoCodigo, 
                                        "NRSEQORCAMENTO"   =>  $proposta, 
                                        "NRORG"            =>  $nrorg
                        );
                         
                        $this->factoryItFormPrecProp->somaContFormPreco($params, $itens["IDUSAMEDIDA"][$ind]);
                    }
                    /* Se o serviço do índice atual for diferente do índice anterior, insere os valores na tabela */
                    else
                    {
                         
                        $novoCodigo = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
                        
                        $arrNewCode[] = $novoCodigo;
                        $params = array("NRITEMORCAMENTO"  => $novoCodigo, 
                                        "NRSEQORCAMENTO"   => $proposta, 
                                        "CDSERVORCA"       => $itens["CDSERVORCA"][$ind], 
                                        "NMITEMORCAMENTO"  => $itens["NMSERVORCA"][$ind],
                                        "QTITEMORCAMENTO"  => $itens["QTCALCULO"][$ind], 
                                        "CDTPMEDIDASERV"   => $itens["CDTPMEDIDASERV"][$ind], 
                                        "QTMEDIDAORCA"     => $itens["QTMEDIDACALC"][$ind],
                                        "VRUNITITEMORCA"   => $itens["VRUNITARIO"][$ind], 
                                        "VRTOTALITEMORCA"  => $itens["VRTOTAL"][$ind], 
                                        "IDMULTMEDVRTOT"   => $itens["IDUSAMEDIDA"][$ind], 
                                        "NRORG"            => $nrorg,
                                        "CDOPERADOR"       => $operador
                        );
                         
                        $this->factoryItFormPrecProp->insereItemContFormPreco($params);
                        $this->inserirRelacionamentoFormacaoPrecoProposta($itens["NRFORMPRECOSERVICO"][$ind], $novoCodigo, $proposta, $nrorg, $operador);
                    }
                }
            }
            
            $this->geraRateios($itens, $proposta, $arrNewCode);
            
            $this->connection->commit();    
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function inserirRelacionamentoFormacaoPrecoProposta($nrformprecoservico, $nritemocontrato, $nrseqorcamento, $nrorg, $cdoperador) {
        $params = array(
            "NRRELACFORMACAOPROP" => $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true),
            "NRFORMPRECOSERVICO"  => $nrformprecoservico,
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRITEMORCAMENTO"     => $nritemocontrato,
            "IDATIVO"             => "S",
            "NRORG"               => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador
        );
        
        $this->factoryItFormPrecProp->inserirRelacionamentoFormacaoPrecoProposta($params);
    }
    
    /**
     *  A função "geraRateios" organiza os arrays para a geração dos arrays e 
     *  dispara a inserção.
     */
    public function geraRateios($itens, $proposta, $arrNewCode){
        
        $nrorg = $this->environment->getNrOrg();
        $codigos = implode(",", $arrNewCode);
        $inseridos = array();
        
        $insAux = $this->factoryItFormPrecProp->retornaItensInseridos($codigos, $proposta, $nrorg);
        
        foreach ($insAux as $ins){
             $serv = $ins["CDSERVORCA"];
             $inseridos[$serv] =  $ins["NRITEMORCAMENTO"];
        };
        
        $this->insereRateios($itens, $proposta, $inseridos);
        
    }
    
   /**
    *  A função "insereRateios" insere os rateios para os itens do contrato
    */
    public function insereRateios($itens, $proposta, $inseridos){
        
        $nrorg = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        
        $count = count($itens["CDSERVORCA"]);
         
        /* for criado para rodar os itens e gerar os ratieos para cada item */   
        for ($i = 0; $i < $count; $i++) {
            /* Chave para buscar os itens do contrato ja inseriudos de acorco com o serviço*/
            $key = $itens["CDSERVORCA"][$i];
            
            /* Fields a serem inseridos*/
            $nrItem = $inseridos[$key];
            $estrutura = $itens["NRESTRUTURAM"][$i];
            $vrRat  = $itens["VRUNITARIO"][$i];
            
            
            if ($estrutura == null){
                
                throw new \Exception ("Operação Bloquada.<br><br> A geração dos rateios não pôde ser realizada pois foi selecionada uma formação de preço sem local de atendimento parametrizado. Favor verificar o cadastro das formações de preço.");
            }
            else{
                /*Para o primeiro índice do For, sempre insere o rateio */
                if ($i == 0){
                
                    $novoCodigo = $this->newCode->getCodeByTableName("FCT_RELACESTRUTURAPROP", $nrorg, 1, 10, true);
                 
                    $params = array("NRRELACESTRUTPROP" => $novoCodigo, 
                                    "NRSEQORCAMENTO"    => $proposta, 
                                    "NRITEMORCAMENTO"   => $nrItem, 
                                    "NRESTRUTURAM"      => $estrutura,
                                    "VRITEMRATEIO"      => $vrRat, 
                                    "NRORG"             => $nrorg,
                                    "CDOPERADOR"        => $operador
                    );
                    
                    $this->factoryItFormPrecProp->insereRateio($params);
                }
                else{
                    /* Valida se, de acordo com o item e a estrutura, já existe um registro criado. 
                     * se existir, atualiza o valor do rateio. Se não existir, insere o rateio.
                    */
                    $existe = $this->buscaRateio($proposta, $nrItem, $estrutura);    
                    
                    if ($existe == true){
                        
                        $params = array("VRITEMRATEIO"      => $vrRat, 
                                        "NRSEQORCAMENTO"    => $proposta, 
                                        "NRITEMORCAMENTO"   => $nrItem, 
                                        "NRESTRUTURAM"      => $estrutura,
                                        "NRORG"             => $nrorg
                        );
                        
                        $this->factoryItFormPrecProp->atualizaRateio($params);
                    }
                    else{
                    
                        $novoCodigo = $this->newCode->getCodeByTableName("FCT_RELACESTRUTURAPROP", $nrorg, 1, 10, true);
                 
                        $params = array("NRRELACESTRUTPROP" => $novoCodigo, 
                                        "NRSEQORCAMENTO"    => $proposta, 
                                        "NRITEMORCAMENTO"   => $nrItem, 
                                        "NRESTRUTURAM"      => $estrutura,
                                        "VRITEMRATEIO"      => $vrRat, 
                                        "NRORG"             => $nrorg,
                                        "CDOPERADOR"        => $operador
                        );
                        
                        $this->factoryItFormPrecProp->insereRateio($params);
                    }
                }
            }
        }
    }
    
    /**
     *  A função "buscaRateio" verifica se ja existe o rateio para o item e estrutura
     */
    public function buscaRateio($proposta, $nrItem, $estrutura){
        $nrorg = $this->environment->getNrOrg();
        
        $params = array("NRSEQORCAMENTO"    => $proposta, 
                        "NRITEMORCAMENTO"   => $nrItem, 
                        "NRESTRUTURAM"      => $estrutura, 
                        "NRORG"             => $nrorg
        );
        
        $return = $this->factoryItFormPrecProp->buscaRateio($params);
        return $return[0]["EXISTE"] != 0;
        
    }
    
    
    
    
    
    
    /* ------------------ ------------------------ ---------------------------------------*/
    
    
    
    /**
     *  A função "buscaIdentValores" busca os identificadores dos elementos e seus respectivos valores
     */
     
     public function buscaIdentValores($nrreajuscontformprec, $IdItemCalculo){
    
        $params = array(
            "NRREAJUSCONTFORMPREC" => $nrreajuscontformprec,
            "IDITEMCALCULO"      => $IdItemCalculo,
            "NRORG"              => $this->environment->getNrOrg()
        );
        
        $IdentValores = $this->reajusteContratoFactory->buscaIdentValores($params);
        return count($IdentValores) > 0 ? $IdentValores[0] : null;
    }
    
    
    /**
     *  A função "recalculoElemento"  inicia os recalculos dos elementos.   
     */
     
    public function recalculoElemento($nrcalcreajformpre) {
        $this->calculados = array();
        $this->recalculoElementoAux($nrcalcreajformpre);
    }
    
    
    /**
     *  A função "recalcularArvore"  Inicia o recalculo de toda a árvore de elementos.
     */
    public function recalcularArvore($nrreajuscontformprec) {
        $this->calculados = array();
        
        $params = array(
            "NRORG"              => $this->environment->getNrOrg(),
            "NRREAJUSCONTFORMPREC" => $nrreajuscontformprec
        );
        
        $elementos = $this->reajusteContratoFactory->getElementosArvoreCalcFormPrev($params);
        foreach ($elementos as $elemento) {
            $this->recalculoElementoAux($elemento["NRCALCREAJFORMPRE"]);
        }
    }
    
    /**
     *  A função "recalculoElemento" realiza o recalculo dos elementos.
     */
    
     public function recalculoElementoAux($nrcalcreajformpre) {
    
        /* 1 - Busca os Elementos para Recálculo  (Somente os do tipo "Somatório dos Filhos" e "Expreessão")*/
        $params = array(
            "NRORG"              => $this->environment->getNrOrg(),
            "NRCALCREAJFORMPRE" => $nrcalcreajformpre
        );
        
        $arrayRecalculo = $this->reajusteContratoFactory->getElementoCalcFormPrec($params);
        $recalculo = $arrayRecalculo[0];
        
        /* 2 - Realiza o Recalculo dos Elementos */        
        $nrreajuscontformprec = $recalculo["NRREAJUSCONTFORMPREC"];
        $nrcalcreajformpre = $recalculo["NRCALCREAJFORMPRE"];
        $quantidade = $recalculo["QTCALCULO"];
        $usaMedida  = $recalculo["IDUSAMEDIDA"];
        $qtMedida   = $recalculo["QTMEDIDACALC"];
        $vrPercentual = $recalculo["VRPERCCALCULO"];
        $idItemCalculo = $recalculo["IDITEMCALCULO"];
        
        if (array_key_exists($idItemCalculo, $this->calculados)) {
            return $this->calculados[$idItemCalculo];
        }
        else {
            /* - Somatório - */
            if ($recalculo["IDTIPOCALCULO"] == "S") {
                
                return $this->somatorioRecalculoCalc($nrcalcreajformpre, $idItemCalculo, $quantidade, $usaMedida, $qtMedida);
            }
            /* - Expressão - */
            else{
            
               $dsExpessaoCalc = $recalculo["DSEXPRESSAOCALC"];
               
               return $this->recalcularExpressao($nrreajuscontformprec, $nrcalcreajformpre, $idItemCalculo, $dsExpessaoCalc, $quantidade, $usaMedida, $qtMedida, $vrPercentual);
                
            } 
        }
        
    }
    /**
     *  A função "somatorioRecalculo" realiza o somatorio dos filhos para recalculo dos elementos.
     */
    public function somatorioRecalculoCalc($nrcalcreajformpre, $idItemCalculo, $quantidade, $usaMedida, $qtMedida){
        if (array_key_exists($idItemCalculo, $this->calculados)) {
            
            return $this->calculados[$idItemCalculo];
        }
        else {
            $params = array(
                "NRCALCREAJFORMPRE" => $nrcalcreajformpre,
                "NRORG" => $this->environment->getNrOrg()
            );
            
            $valorUnitario = 0;
            $somatorioPercentual = 0;
            
            $elementosFilhos = $this->reajusteContratoFactory->getElementosFilhosCalcFormPrec($params);
            foreach ($elementosFilhos as $elemento) {
                if (array_key_exists($elemento["IDITEMCALCULO"], $this->calculados)) {
                    $valorUnitario += $this->calculados[$elemento["IDITEMCALCULO"]]["VRTOTAL"];
                    $somatorioPercentual += $this->calculados[$elemento["IDITEMCALCULO"]]["VRPERCCALCULO"];
                }
                else {
                    if ($elemento["IDTIPOCALCULO"] == "V") {
                        $valorUnitario += (float)$elemento["VRTOTAL"];
                        $somatorioPercentual += (float)$elemento["VRPERCCALCULO"];
                    }
                    else if ($elemento["IDTIPOCALCULO"] == "S") {
                        $resultado = $this->somatorioRecalculoCalc($elemento["NRCALCREAJFORMPRE"], $elemento["IDITEMCALCULO"], $elemento["QTCALCULO"], $elemento["IDUSAMEDIDA"], $elemento["QTMEDIDACALC"]);
                        
                        $valorUnitario += $resultado["VRTOTAL"];
                        $somatorioPercentual += $resultado["VRPERCCALCULO"];
                    }
                    else {
                        $resultado = $this->recalcularExpressao($elemento["NRREAJUSCONTFORMPREC"], $elemento["NRCALCREAJFORMPRE"], $elemento["IDITEMCALCULO"], $elemento["DSEXPRESSAOCALC"], $elemento["QTCALCULO"], $elemento["IDUSAMEDIDA"], $elemento["QTMEDIDACALC"], $elemento["VRPERCCALCULO"]);
                        
                        $valorUnitario += $resultado["VRTOTAL"];
                        $somatorioPercentual += $resultado["VRPERCCALCULO"];
                    }
                }
            }
            
            $valorTotal = $valorUnitario * $quantidade;
            if ($usaMedida == "S") {
                $valorTotal *= $qtMedida;
            }
            
            $this->calculados[$idItemCalculo]["VRTOTAL"] = $valorTotal;
            $this->calculados[$idItemCalculo]["VRPERCCALCULO"] = $somatorioPercentual;
            
            /* Salva o recálculo do Elemento */
            $this->salvarRecalculo($nrcalcreajformpre, $valorUnitario, $quantidade, $qtMedida, $valorTotal, $somatorioPercentual);  
            
            return $this->calculados[$idItemCalculo];
        }
    }
    
     
    /**
     *  A função "recalcularExpressao" realiza o recalculo dos valores para elementos do tipo "Expressão".
     */
    public function recalcularExpressao($nrreajuscontformprec, $nrcalcreajformpre, $idItemCalculo, $expressao, $quantidade, $usaMedida, $qtMedida, $vrPercentual) {
        
        if (array_key_exists($idItemCalculo, $this->calculados)) {
            
            return $this->calculados[$idItemCalculo];
        }
        else {
            $expressao = str_replace(array("[", "]"), "@@", $expressao);
            $ids = explode("@@", $expressao);
            
            for ($i = 1; $i < count($ids); $i = $i + 2) {
                
                $elementoExp = $this->buscaIdentValores($nrreajuscontformprec, $ids[$i]);
                if (!empty($elementoExp)) {
                
                    $nrcalcreajformpreElementoExp = $elementoExp["NRCALCREAJFORMPRE"];
                    $vrIdTipoCalculoElementoExp    = $elementoExp["IDTIPOCALCULO"];
                    $idItemCalculoElementoExp      = $elementoExp["IDITEMCALCULO"];
                    $expressaoElementoExp          = $elementoExp["DSEXPRESSAOCALC"];
                    $percentualElementoExp         = $elementoExp["VRPERCCALCULO"];
                    $quantidadeElementoExp         = $elementoExp["QTCALCULO"];
                    $usaMedidaElementoExp          = $elementoExp["IDUSAMEDIDA"];
                    $qtMedidaElementoExp           = $elementoExp["QTMEDIDACALC"];
                    
                    if ($vrIdTipoCalculoElementoExp == "E") {
                        
                        $resultado = $this->recalculoElementoAux($nrcalcreajformpreElementoExp);
                        $valorTotalElementoExp = $resultado["VRTOTAL"];
                        
                    }
                    else if ($vrIdTipoCalculoElementoExp == "S") {
                        $resultado = $this->somatorioRecalculoCalc ($nrcalcreajformpreElementoExp, $idItemCalculoElementoExp, $quantidadeElementoExp, $usaMedidaElementoExp, $qtMedidaElementoExp);
                        
                        $valorTotalElementoExp = $resultado["VRTOTAL"];
                
                    }
                    else {
                        $valorTotalElementoExp = $elementoExp["VRTOTAL"];
                    }
                    
                    $expressao = str_replace("@@".$ids[$i]."@@", $valorTotalElementoExp, $expressao);
                }
            }
            
            /* Realiza o cálculo do elemento */    
            $vrUnitario = eval('return ' . $expressao . ';');
            
            if (!empty($vrPercentual)) {
                $vrUnitario = $vrUnitario * $vrPercentual / 100;
            }
            
            $vrTotal = $vrUnitario * $quantidade;
            if ($usaMedida == 'S') {
                $vrTotal *= $qtMedida; 
            }
            
            
            $this->calculados[$idItemCalculo]["VRTOTAL"] = $vrTotal;
            $this->calculados[$idItemCalculo]["VRPERCCALCULO"] = $vrPercentual;
            
            /* Salva o recálculo do Elemento */
            $this->salvarRecalculo($nrcalcreajformpre, $vrUnitario, $quantidade, $qtMedida, $vrTotal, $vrPercentual);
            
            
            return $this->calculados[$idItemCalculo];
        }
    }
    
    
    /**
     *  A função "salvarRecalculo" salva o recalculo dos elementos.
     */
     
     public function salvarRecalculo($nrcalcreajformpre, $vrUnitario, $qtCalculo, $qtmedida, $vrTotal, $vrPercentual){
    
        $params = array(
            "NRCALCREAJFORMPRE" => $nrcalcreajformpre,
            "VRUNITARIO"         => $vrUnitario,
            "QTCALCULO"          => $qtCalculo,
            "QTMEDIDACALC"       => $qtmedida,
            "VRTOTAL"            => $vrTotal,
            "VRPERCCALCULO"      => $vrPercentual,
            "NRORG"              => $this->environment->getNrOrg()
        );
        
        $this->reajusteContratoFactory->salvarRecalculo($params); 
        
    }
    
    
}
?>